/*******************************************************************************
 * Copyright (c) 2004 - 2006 University Of British Columbia and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     University Of British Columbia - initial API and implementation
 *******************************************************************************/

package org.eclipse.mylar.internal.tasks.ui.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.mylar.context.core.ContextCorePlugin;
import org.eclipse.mylar.context.core.MylarStatusHandler;
import org.eclipse.mylar.internal.context.core.MylarContextManager;
import org.eclipse.mylar.internal.tasks.ui.ITasksUiConstants;
import org.eclipse.mylar.tasks.core.AbstractTaskContainer;
import org.eclipse.mylar.tasks.core.ITask;
import org.eclipse.mylar.tasks.core.ITaskListChangeListener;
import org.eclipse.mylar.tasks.ui.TaskListManager;
import org.eclipse.mylar.tasks.ui.TasksUiPlugin;
import org.eclipse.ui.PlatformUI;

/**
 * @author Mik Kersten
 * @author Eugene Kuleshov
 */
public class TaskListSaveManager implements ITaskListChangeListener, IBackgroundSaveListener {

	private final static int DEFAULT_SAVE_INTERVAL = 5 * 60 * 1000;

	private BackgroundSaveTimer saveTimer;

	private TaskListSaverJob taskListSaverJob;

	private boolean initializationWarningDialogShow = false;

	/**
	 * Fort testing.
	 */
	private boolean forceBackgroundSave = false;

	public TaskListSaveManager() {
		saveTimer = new BackgroundSaveTimer(this);
		saveTimer.setSaveIntervalMillis(DEFAULT_SAVE_INTERVAL);
		saveTimer.start();
		
		taskListSaverJob = new TaskListSaverJob();
		taskListSaverJob.schedule();
	}

	/**
	 * Called periodically by the save timer
	 */
	public void saveRequested() {
		if (TasksUiPlugin.getDefault() != null && TasksUiPlugin.getDefault().isShellActive() || forceBackgroundSave) {
			try {
				saveTaskList(true, true);
			} catch (Exception e) {
				MylarStatusHandler.fail(e, "Could not auto save task list", false);
			}
		}
	}

	public void saveTaskList(boolean saveContext) {
		saveTaskList(saveContext, false);
	}
	
	public void saveTaskList(boolean saveContext, boolean async) {
		if (TasksUiPlugin.getDefault() != null && TasksUiPlugin.getDefault().isInitialized()) {
			TaskListManager taskListManager = TasksUiPlugin.getTaskListManager();
			if(async) {
				if (saveContext) {
					for (ITask task : taskListManager.getTaskList().getActiveTasks()) {
						taskListSaverJob.addTaskContext(task);
					}
				}
				taskListSaverJob.requestSave();
			} else {
				taskListSaverJob.waitSaveCompleted();
				MylarContextManager contextManager = ContextCorePlugin.getContextManager();
				if (saveContext) {
					for (ITask task : new ArrayList<ITask>(taskListManager.getTaskList()
							.getActiveTasks())) {
						contextManager.saveContext(task.getHandleIdentifier());
					}
				}
				internalSaveTaskList();
			}
		} else if (PlatformUI.getWorkbench() != null && !PlatformUI.getWorkbench().isClosing()) {
			MylarStatusHandler.log("Possible task list initialization failure, not saving list.", this);
			if (!initializationWarningDialogShow) {
				initializationWarningDialogShow = true;
				PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
					public void run() {
						if (PlatformUI.getWorkbench() != null && PlatformUI.getWorkbench().getDisplay() != null) {
							MessageDialog
									.openInformation(
											PlatformUI.getWorkbench().getDisplay().getActiveShell(),
											ITasksUiConstants.TITLE_DIALOG,
											"If task list is blank, Mylar Task List may have failed to initialize.\n\n"
													+ "First, try restarting to see if that corrects the problem.\n\n"
													+ "Then, check the Error Log view for messages, and the FAQ for solutions.\n\n"
													+ ITasksUiConstants.URL_HOMEPAGE);
						}
					}
				});
			}
		}
	}
	
	private void internalSaveTaskList() {
		TaskListManager taskListManager = TasksUiPlugin.getTaskListManager();
		taskListManager.getTaskListWriter().writeTaskList(taskListManager.getTaskList(),
				taskListManager.getTaskListFile());
	}

	/**
	 * Copies all files in the current data directory to the specified folder.
	 * Will overwrite.
	 */
	public void copyDataDirContentsTo(String targetFolderPath) {
		saveTaskList(true, false);
		
		File mainDataDir = new File(TasksUiPlugin.getDefault().getDataDirectory());

		for (File currFile : mainDataDir.listFiles()) {
			if (currFile.isFile()) {
				File destFile = new File(targetFolderPath + File.separator + currFile.getName());
				copy(currFile, destFile);
			} else if (currFile.isDirectory()) {
				File destDir = new File(targetFolderPath + File.separator + currFile.getName());
				if (!destDir.exists()) {
					if (!destDir.mkdir()) {
						MylarStatusHandler.log("Unable to create destination context folder: "
								+ destDir.getAbsolutePath(), this);
						continue;
					}
				}
				for (File file : currFile.listFiles()) {
					File destFile = new File(destDir, file.getName());
					if (destFile.exists()) {
						destFile.delete();
					}
					copy(file, destFile);
				}
			}
		}
	}

	// public void createTaskListBackupFile() {
	// String path = TasksUiPlugin.getDefault().getDataDirectory() +
	// File.separator
	// + TasksUiPlugin.DEFAULT_TASK_LIST_FILE;
	// File taskListFile = new File(path);
	// String backup = path.substring(0, path.indexOf('.')) +
	// FILE_SUFFIX_BACKUP;
	// copy(taskListFile, new File(backup));
	// }
	//
	// public String getBackupFilePath() {
	// String path = TasksUiPlugin.getDefault().getDataDirectory() +
	// File.separator
	// + TasksUiPlugin.DEFAULT_TASK_LIST_FILE;
	// return path.substring(0, path.indexOf('.')) + FILE_SUFFIX_BACKUP;
	// }
	//
	// public void reverseBackup() {
	// String path = TasksUiPlugin.getDefault().getBackupFolderPath() +
	// File.separator
	// + TasksUiPlugin.DEFAULT_TASK_LIST_FILE;
	// File taskListFile = new File(path);
	// String backup = path.substring(0, path.indexOf('.')) +
	// FILE_SUFFIX_BACKUP;
	// copy(new File(backup), taskListFile);
	// }

	private boolean copy(File src, File dst) {
		try {
			InputStream in = new FileInputStream(src);
			OutputStream out = new FileOutputStream(dst);

			// Transfer bytes from in to out
			byte[] buf = new byte[1024];
			int len;
			while ((len = in.read(buf)) > 0) {
				out.write(buf, 0, len);
			}
			in.close();
			out.close();
			return true;
		} catch (IOException ioe) {
			return false;
		}
	}

	public void taskActivated(ITask task) {
		// ignore
	}

	public void tasksActivated(List<ITask> tasks) {
		// ignore
	}

	public void taskDeactivated(ITask task) {
		saveTaskList(true, true);
	}

	public void localInfoChanged(ITask task) {
		saveTaskList(false, true);
	}

	public void repositoryInfoChanged(ITask task) {
		// ignore
	}

	public void tasklistRead() {
		// ignore
	}

	/**
	 * For testing.
	 */
	public void setForceBackgroundSave(boolean on) {
		forceBackgroundSave = on;
		saveTimer.setForceSyncExec(on);
	}

	public void taskMoved(ITask task, AbstractTaskContainer fromContainer, AbstractTaskContainer toContainer) {
		saveTaskList(false, true);
	}

	public void taskDeleted(ITask task) {
		saveTaskList(false, true);
	}

	public void containerAdded(AbstractTaskContainer container) {
		saveTaskList(false, true);
	}

	public void containerDeleted(AbstractTaskContainer container) {
		saveTaskList(false, true);
	}

	public void taskAdded(ITask task) {
		saveTaskList(false, true);
	}

	/** For testing only * */
	public BackgroundSaveTimer getSaveTimer() {
		return saveTimer;
	}

	public void containerInfoChanged(AbstractTaskContainer container) {
		saveTaskList(false, true);
	}

	public void synchronizationCompleted() {
		// ignore
	}
	
	private class TaskListSaverJob extends Job {

		private final Queue<ITask> taskQueue = new LinkedList<ITask>();

		private volatile boolean saveRequested = false;

		private volatile boolean saveCompleted = true; 
		
		
		TaskListSaverJob() {
			super("Task List Saver");
			setPriority(Job.LONG);
			setSystem(true);
		}

		protected IStatus run(IProgressMonitor monitor) {
			while(true) {
				if(saveRequested) {
					saveRequested = false;
					saveCompleted = false;
					MylarContextManager contextManager = ContextCorePlugin.getContextManager();
					while(!taskQueue.isEmpty()) {
						ITask task = taskQueue.poll();
						if(task!=null) {
							contextManager.saveContext(task.getHandleIdentifier());
						}
					}
					internalSaveTaskList();
				}
				
				if(!saveRequested) {
					synchronized (this) {
						saveCompleted = true;
						notifyAll();
						try {
							wait();
						} catch (InterruptedException ex) {
							// ignore
						}
					}
				}
			}
		}
		
		void addTaskContext(ITask task) {
			taskQueue.add(task);
		}
		
		void requestSave() {
			saveRequested = true;
			synchronized (this) {
				notifyAll();
			}
		}

		void waitSaveCompleted() {
			while(!saveCompleted) {
				synchronized (this) {
					try {
						wait();
					} catch (InterruptedException ex) {
						// ignore
					}
				}
			}
		}
	}
	
}
